!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! geofold_global.f90
!!! This file contains three modules: geofold_global, geofold_masker, geofold_hbonds
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
MODULE geofold_global
  private
  !!--------- global types and parameters for GEOFOLD ------------!!
  !! INTEGER, PARAMETER :: maxres = 80 !maximum number of residues
  INTEGER, PARAMETER :: MAXRES=1600
  integer,parameter :: MAXCHAIN=26  !! Maximum number of separate chains.
  integer,save :: maxsplit=8
  real,parameter ::  FULSPLITDEPTH=4.  !! recursion depth limit for maximum splitting.
  real,parameter :: MAXHINGEANG=30.0 ! atoms must hinge at least this much
  character,dimension(MAXCHAIN) :: chainletters=(/"A","B","C","D","E","F","G","H","I","J","K","L","M","N",&
                                                  "O","P","Q","R","S","T","U","V","W","X","Y","Z"/)
  TYPE intermediate
     INTEGER :: idnum                          ! unique number identifier of the intermediate state
     CHARACTER, dimension(maxres) :: iflag     ! flags that define the intermediate
     TYPE (intermediate), POINTER :: next       ! next intermediate
     INTEGER :: state                           ! breakflag, pivotflag, hingeflag or meltflag
     integer :: sym                             ! copy number for symmetric proteins
     integer :: axis                            ! index of vectorball
  END TYPE intermediate
  TYPE tstate
     INTEGER :: id                              ! id of the tstate
     INTEGER :: tp                              ! type of the tstate (hinge, pivot, etc.)
     REAL :: entropy                            ! entropy value 0.-1.
     integer :: parent, child1, child2        ! idnum from type (intermediate)
     TYPE (tstate), POINTER :: next             ! next tstate
     REAL :: energy                             !
     integer :: axis                            ! index to vectorball, axis of + rotation
  END TYPE tstate
  TYPE (intermediate), POINTER :: ilistroot   ! declare global intermediate list
  TYPE (tstate), POINTER :: tlistroot         ! declare global tstate list
  REAL, dimension(3, maxres) :: allcoords      ! contain all the c-a coordinates
  REAL :: nativeenergy                         ! contains information about native state
  TYPE (intermediate), POINTER :: iptr         ! pointer for ilistroot (for later)
  CHARACTER, dimension(maxres) :: masterchains ! contains all the chain IDs
  integer,dimension(maxres) :: seq, resseq
  INTEGER,parameter ::  breakflag=1, pivotflag=2, hingeflag=3, meltflag=4
  INTEGER ::  geofold_nres, geofold_split
  logical :: verbose=.false.
  real :: hcutoff=0.5, bcutoff=0.2, scutoff=0.01
  !!
  public :: geofold_nres, geofold_split, intermediate, tstate, ilistroot
  public :: tlistroot, allcoords, masterchains, breakflag
  public :: pivotflag, hingeflag, meltflag, verbose, hcutoff, bcutoff, scutoff
  public :: MAXRES, MAXCHAIN, maxsplit, MAXHINGEANG, FULSPLITDEPTH
  public :: seq, resseq, chainletters, geofold_pickunit
  public :: geofold_readpdb, geofold_writepdb
CONTAINS
  !-----------------------------------------------------------------------------
  integer function aa2num(aa)
    character(len=3),intent(in) :: aa
    character(len=80),parameter :: allaa="ALA CYS ASP GLU PHE GLY HIS ILE LYS LEU MET ASN PRO GLN ARG SER THR VAL TRP TYR "
    aa2num = index(allaa,aa)/4 + 1
  end function aa2num
!!====================================================================================
  integer function geofold_pickunit(iunit)
    implicit none
    integer,intent(in) :: iunit
    logical :: alreadyused
    integer :: ounit
    ounit = iunit
    inquire(unit=ounit,opened=alreadyused)
    do while (alreadyused)
      ounit = ounit + 1
      inquire(unit=ounit,opened=alreadyused)
    enddo
    geofold_pickunit = ounit
  end function geofold_pickunit
!!====================================================================================
  SUBROUTINE geofold_readpdb(dunit)
  implicit none
  integer,intent(in) :: dunit
  ! character(len=*),intent(in) :: filename
  integer ::  ierr, ires
  character(len=1000) :: aline
  character :: chainID
  character(len=3) :: aa
  !!
  !! masterchains is global
  masterchains = '.'
  ires = 0
  DO
     read(dunit,'(a)', iostat=ierr) aline !read in pdb file
     IF ( ierr /=0 ) EXIT 
     IF (aline(1:5) /= "ATOM ".and.aline(1:7) /= "HETATM ") CYCLE
     IF (aline(13:16) /=" CA " ) CYCLE
     ires = ires + 1
     ! Get Coordinates and Chain ID (_ 'Char')
     ChainID = aline(22:22) 
     IF ( ChainID == ' ') THEN
        masterchains(ires) = 'b'
     ELSE 
        masterchains(ires) = ChainID
     END IF
     !! diagnostic
     ! write (*,*) trim(aline)
     read( aline(31:54),'(3f8.3)' ), allcoords(1:3, ires)   
     aa = aline(18:20)
     seq(ires) = aa2num(aa)
     read( aline(23:26),'(i4)' ) resseq(ires)
     !! diagnostic
     ! write(*,*) "                     ",ires,aa,seq(ires),resseq(ires),allcoords(1:3, ires)
  END DO 
  geofold_nres = ires
  END SUBROUTINE geofold_readpdb
  !----------------------------------------------------------------------------------
  SUBROUTINE geofold_writepdb(dunit)
    integer,intent(in) :: dunit
    character(len=80) :: aline
    integer :: ires, ios
    character(len=80) :: aa="ALA CYS ASP GLU PHE GLY HIS ILE LYS LEU MET ASN PRO GLN ARG SER THR VAL TRP TYR "
    aline = "ATOM      2  CA  MET A   1       2.666  13.148  30.331  1.00 55.69"
    !--------123456789012345678901234567890123456789012345678901234567890
    do ires=1,geofold_nres
      write(aline(8:11),'(i4)',iostat=ios) ires; if (ios/=0) stop 'BUG ires'
      aline(18:20) = aa((seq(ires)-1)*4+1:seq(ires)*4-1)
      if (masterchains(ires)=='b') then
        aline(22:22) = ' '
      else
        aline(22:22) = masterchains(ires)
      endif
      write(aline(23:26),'(i4)',iostat=ios) resseq(ires); if (ios/=0) stop 'BUG ires'
      write(aline(31:54),'(3f8.3)',iostat=ios) allcoords(1:3,ires)
      if (ios/=0) stop 'BUG allcoords'
      write(aline(55:66),'(2f6.2)' ) 0.0, 0.0 ; if (ios/=0) stop 'BUG B'
      write(dunit,'(a)',iostat=ios) trim(aline) ; if (ios/=0) stop 'BUG aline'
    enddo
  END SUBROUTINE geofold_writepdb
  !----------------------------------------------------------------------------------
END MODULE geofold_global


!!====================================================================================
!! geofold_hbonds
!!====================================================================================
MODULE geofold_hbonds
 use geofold_global
 private
 integer, dimension(:,:), allocatable :: geofold_hbonds_hb
 integer, dimension(:), allocatable :: geofold_hbonds_ss
 integer ::  nhbonds
 public ::  geofold_hbonds_get, geofold_hbonds_read, geofold_hbonds_cleanup
 public ::  geofold_hbonds_disulfidepresent, geofold_hbonds_ssinseg
 CONTAINS
!!====================================================================================
! get_hbonds
! count the number of hbonds within intemedaite f
!
  subroutine geofold_hbonds_get(f, hbonds)
    implicit none
    type(intermediate), POINTER :: f
    integer,intent(out) :: hbonds
    integer :: n,ires,jres
    n = 0
    ILOOP: do ires=1,geofold_nres
      if (f%iflag(ires)==".") cycle ILOOP
      jres = geofold_hbonds_hb(1,ires) ! jres is donor to ires
      if (jres/=0) then
        if (f%iflag(jres)/=".") then
          n = n + 1
        endif
      endif
      jres = geofold_hbonds_hb(2,ires) ! jres is acceptor to ires
      if (jres/=0) then
        if (f%iflag(jres)/=".") then
          n = n + 1
        endif
      endif
    enddo ILOOP
    hbonds = n/2
  end subroutine geofold_hbonds_get

 !!----------------------
 !! read Hbonds file
 SUBROUTINE geofold_hbonds_read(mfile)
  implicit none
  ! integer,intent(in) :: harg
  CHARACTER (len=*) :: mfile
  INTEGER :: res1, res2, ierr, dunit, n, ss
  REAL :: engy, x
  CHARACTER (len=200) :: aline
  character :: bond !! "H" or "S" for Hbond or disulfide
  integer :: minres, maxres
  dunit = 34
  minres = 999
  maxres = -999
  ! call getarg(harg, mfile)
  if (allocated(geofold_hbonds_hb)) deallocate(geofold_hbonds_hb)
  allocate(geofold_hbonds_hb(2, geofold_nres),stat=ierr)
  if (ierr/=0) stop 'read_hbonds:: error allocating geofold_hbonds_hb'
  allocate(geofold_hbonds_ss(geofold_nres),stat=ierr)
  if (ierr/=0) stop 'read_hbonds:: error allocating geofold_hbonds_ss'
  dunit = geofold_pickunit(dunit)
  open(dunit, file = mfile, iostat=ierr, status="old", form="formatted")
  IF (ierr /= 0 ) STOP "geofold:: read_hbondfile:  error opening file!"
  geofold_hbonds_hb = 0.
  geofold_hbonds_ss = 0.
  n = 0
  ss = 0
  DO
     !! res1 is dono, res2 is acceptor
     !! geofold_hbond_hb(1,i) is donor to i
     !! geofold_hbond_hb(2,i) is acceptor to i
     read(dunit,'(a)',iostat=ierr) aline
     IF (ierr /=0) EXIT
     if (aline(1:1)=='!') cycle
     read(aline,*, iostat=ierr) res1, res2, bond ! currently engy is ignored.
     IF (ierr /=0) EXIT
     select case (bond)
     case ("H","h") 
       geofold_hbonds_hb(1,res2) = res1
       geofold_hbonds_hb(2,res1) = res2
       n = n + 1
     case ("S","s")
       geofold_hbonds_ss(res1) = res2
       geofold_hbonds_ss(res2) = res1
       ss = ss + 1
     end select
     if (res1 < minres) minres = res1
     if (res2 < minres) minres = res2
     if (res1 > maxres) maxres = res1
     if (res2 > maxres) maxres = res2
  END DO
  close(dunit) ! close hbonds file
  write(*,'("Number of HBONDs read from ",a,i8)') trim(mfile), n
  write(*,'("Number of SS-bonds read from ",a,i8)') trim(mfile), ss
  write(*,'("HBOND contact data found for residue range ",2i8)') minres, maxres
  write(*,'("Last Hbond  ",2i8,f9.4)') res1,res2,engy
END SUBROUTINE geofold_hbonds_read

!!========================================================================
  logical function geofold_hbonds_ssinseg(ires,jres)
    implicit none
    integer,intent(in) :: ires,jres
    integer :: i
    geofold_hbonds_ssinseg = .false.
    do i=ires,jres
      if (geofold_hbonds_ss(i)/=0) then
        geofold_hbonds_ssinseg = .true.
        return
      endif
    enddo
    return
  end function geofold_hbonds_ssinseg
!!========================================================================
  logical function geofold_hbonds_disulfidepresent(u1,u2,nres)
    !! Ask whether a disulfide connects u1 and u2.
    implicit none
    integer,intent(in) :: nres
    character,dimension(nres),intent(in) :: u1,u2
    integer :: i,j
    geofold_hbonds_disulfidepresent = .false.
    do i=1,nres
      if (u1(i)==".") cycle
      if (geofold_hbonds_ss(i)==0) cycle
      do j=1,nres
        if (u2(j)==".") cycle
        if (geofold_hbonds_ss(j)==i) then
          geofold_hbonds_disulfidepresent = .true.
          return
        endif
      enddo
    enddo
    return
  end function geofold_hbonds_disulfidepresent
!!====================================================================================
  subroutine geofold_hbonds_cleanup()
    if (allocated(geofold_hbonds_hb)) deallocate(geofold_hbonds_hb)
    if (allocated(geofold_hbonds_ss)) deallocate(geofold_hbonds_ss)
  end subroutine geofold_hbonds_cleanup
!!====================================================================================
end MODULE geofold_hbonds

!!====================================================================================
!! geofold_masker
!!====================================================================================
MODULE geofold_masker
 use geofold_global
 use geofold_hbonds
 private
 REAL, dimension(:,:), allocatable :: sasnrg !contains energies of all the c-a residues
 real,dimension(20)::lambda=(/ &
      0.00, 0.55, 1.25, 1.81, 0.58, 0.00, 0.96, 0.89, 1.94, 0.78, &
      1.61, 1.57, 0.00, 2.11, 2.03, 1.71, 1.63, 0.51, 0.97, 0.98/)
 real,dimension(20)::maxsas=(/ &
      107.05,136.83,150.35,170.56,211.48,90.30,168.59,154.89,176.46,156.33, &
      168.88,149.62,127.45,203.74,200.29,120.02,137.17,136.02,205.44,191.31/)
 REAL, dimension(3, maxres) :: voidcoords      
 type voidtype
     INTEGER :: nn
     integer,dimension(100) :: wall
 end type voidtype
 type(voidtype),dimension(maxres) :: avoid
 integer :: geofold_masker_nvoid
 public :: geofold_masker_energy , geofold_masker_read, geofold_masker_intermediates
 public :: geofold_masker_setvoids, geofold_masker_readvoids
CONTAINS
  !-----------------------------------------------------------------------------
  SUBROUTINE geofold_masker_readvoids(dunit)
    implicit none
    integer,intent(in) :: dunit
    character(len=1000) :: aline
    integer :: ierr, ivoid 
    !!
    !! Get voids from PDB file, if present
    rewind(dunit)
    ivoid = 0
    DO
      read(dunit,'(a)', iostat=ierr) aline !read in pdb file
      IF (ierr /=0 ) EXIT 
      IF (aline(1:7) /= "HETATM ") CYCLE
      IF (aline(18:20) /="HOH" ) CYCLE
      IF (aline(22:22) /="V" ) CYCLE
      ivoid = ivoid + 1
      ! Get Coordinates and Chain ID (_ 'Char')
      read( aline(31:54),'(3f8.3)' ), voidcoords(1:3, ivoid)   
      !! diagnostic
      ! write(*,*) "VOID                     ",voidcoords(1:3, ivoid)
    END DO 
    geofold_masker_nvoid = ivoid
  END SUBROUTINE geofold_masker_readvoids
  !-----------------------------------------------------------------------------

  SUBROUTINE geofold_masker_read(mfile)
    ! integer,intent(in) :: marg
    INTEGER :: res1, res2, ierr, dunit
    REAL :: engy, x
    CHARACTER (len=*) :: mfile
    integer :: minres, maxres
    ! WRITE(0,*) 'in read_maskerfile',"  geofold_nres=",geofold_nres
    dunit = 33
    minres = 999
    maxres = -999
    ! call getarg(marg, mfile)
    allocate(sasnrg(geofold_nres, geofold_nres),stat=ierr)
    if (ierr/=0) stop 'read_maskerfile:: error allocating sasnrg'
    dunit = geofold_pickunit(dunit)
    open(dunit, file = mfile, iostat=ierr, status="old", form="formatted")
    IF (ierr /= 0 ) then
      write(0,*) "geofold_global:: error opening masker file! ", trim(mfile)
      STOP "geofold_global:: error opening masker file!"
    endif
    sasnrg = 0.
    DO
       read(dunit,*, iostat=ierr) res1, res2, engy, x !read in masker file
       IF (ierr /=0) EXIT
       sasnrg(res1, res2) = engy !store information in data structure
       if (res1 < minres) minres = res1
       if (res2 < minres) minres = res2
       if (res1 > maxres) maxres = res1
       if (res2 > maxres) maxres = res2
    END DO
    close(dunit) ! close masker file
    write(*,'("MASKER SAS contact data found for residue range ",2i8)') minres, maxres
    write(*,'("Last data point  ",2i8,f9.4)') res1,res2,engy
  END SUBROUTINE geofold_masker_read
  !!--------------------------------------------------------------
  SUBROUTINE geofold_masker_energy(f, energy)
    ! this involves looking at all pairs of residues in that intermediate
    ! Total Energy = Summation over all i and j
    type(intermediate), POINTER :: f
    REAL,intent(out) :: energy
    REAL :: sumenergy
    INTEGER :: i, j, k
    energy = 0
    sumenergy = 0
    k = 0
    DO i = 1, geofold_nres
       IF ( f%iflag(i) /= '.' ) THEN ! look at intermediate f: f%iflag
          k = k + 1
          DO j = i, geofold_nres ! calculate the energy value
             IF ( f%iflag(j) /= '.' ) THEN 
                sumenergy = sumenergy + sasnrg(i, j)
             END IF
          END DO
       END IF
    END DO
    energy = sumenergy
    !! diagnostcic
    ! write(*,'("energy for ",i8,f12.6,i8)') f%idnum, energy, k
  END SUBROUTINE geofold_masker_energy
!!====================================================================================
!
  subroutine geofold_masker_setvoids()
    implicit none
    integer :: ivoid,i,j,k,ires
    real :: dd
    real,parameter :: dcut=7.0
    !!
    do ivoid=1,geofold_masker_nvoid
      avoid(ivoid)%nn = 0
    enddo
    do ivoid=1,geofold_masker_nvoid
      do ires=1,geofold_nres
        dd = sqrt(sum((allcoords(1:3,ires)-voidcoords(1:3,ivoid))**2))
        if (dd < dcut) then
          avoid(ivoid)%nn = avoid(ivoid)%nn + 1
          avoid(ivoid)%Wall(avoid(ivoid)%nn) = ires
        endif
      enddo
    enddo
  end subroutine geofold_masker_setvoids

  !!------------------------------------------
  SUBROUTINE geofold_masker_intermediates(dunit)
    implicit none
    integer,intent(in) :: dunit
    REAL :: returnedenergy, scenergy
    integer ::  voids=0
    type(intermediate), POINTER :: itemp
    integer :: nres, ierr, hbonds
    nres = geofold_nres
    !!
    ! call geofold_masker_read(2)
    ! call geofold_hbonds_read(3)
    !!
    itemp => ilistroot
    WRITE(dunit, '(a)') 'HEADER ISEGMT isegmt_no recursion_depth symops SAS sc_entropy Voids Hbonds '
    DO while ( ASSOCIATED(itemp%next) ) 
       itemp => itemp%next
       call geofold_masker_energy(itemp, returnedenergy)
       call getscenergy(itemp, scenergy)
       call getvoids(itemp, voids)
       call geofold_hbonds_get(itemp, hbonds)
       !---- print out intermediate information and associated energy
       WRITE(dunit, '(a,i7,2i5,2f12.2,i8,i6)') 'ISEGMT ', itemp%idnum, itemp%state, itemp%sym, &
                                           returnedenergy, scenergy, voids, hbonds
       WRITE(dunit, '(2000(a1))') itemp%iflag(1:geofold_nres)
    END DO
    
    !!
    deallocate(sasnrg,stat=ierr)
    if (ierr/=0) stop 'Error deallocating!!'
    call geofold_hbonds_cleanup()
    !!
  CONTAINS
!!====================================================================================
! getvoids
! count the number of voids that have all of their neighbors
 
     subroutine getvoids(f, voids)
       implicit none
       type(intermediate), POINTER :: f
       integer,intent(out) :: voids
       integer :: i,j,k,n
       n = 0
       ILOOP: do i=1,geofold_masker_nvoid
         if (avoid(i)%nn == 0) cycle
         do j=1,avoid(i)%nn
           if (avoid(i)%wall(j)<=0) stop "BUG in getvoids"
           if (f%iflag(avoid(i)%wall(j))=='.') cycle ILOOP
         enddo
         n = n + 1
       enddo ILOOP
       voids = n
     end subroutine getvoids

  END SUBROUTINE geofold_masker_intermediates 
  
  !!-----------------------------------------------------
  subroutine getscenergy(f, scenergy)   
    implicit none
    type(intermediate), POINTER :: f
    real,intent(out) :: scenergy
    real :: sumsas, sumi, saszero
    integer :: i, j
    scenergy = 0.
    DO i = 1, geofold_nres
      sumsas = 0.
      sumi = 0.
      DO j = 1, geofold_nres ! calculate the energy value
        IF (f%iflag(i)/='.'.and.f%iflag(j)/='.') then 
          sumi = sumi + sasnrg(i, j)
        END IF
        sumsas = sumsas + sasnrg(i, j)
      END DO
      saszero = maxsas(seq(i)) - maxsas(6)  !! max sidechain SAS
      if (saszero > 0.00000001) then
         scenergy = scenergy + lambda(seq(i))*(sumi/saszero)
      endif
    END DO
  end subroutine getscenergy
!!====================================================================================
END MODULE geofold_masker
