!! ========= PDB2HB ============
!! Find backbone H-bonds. Output
!! donor and acceptor id numbers :  idonor iacceptor value
!! Number is sequential as used in GeoFOLD programs.
!! Input PDB must be "clean". Use getchain.f90
!! value is not used, yet.
!! Modified from hbcontactmap.f90
!! --------- in more detail ----------
!! READ PDB file  (output from getchain.f90)
!!   Loop over all residues
!!     For backbone N
!!       Find nearest backbone O under 3.6A
!!       save residue number
!!   enddo
!! -----------------------------------
!! C.Bystroff Sat Nov  8 16:00:14 EST 2008
!!  hbcontactmap.f90 C.Bystroff Thu Apr 13 06:14:50 EDT 2006
!! -----------------------------------
program pdb2hb
implicit none
integer :: iargc, jarg, ios, n,m
character(len=80) :: pdbfile
integer :: nres=0, nsg=0
real,dimension(:,:),pointer :: xyz
real,dimension(:,:),pointer :: sg

!chain = '?'
jarg = iargc()
if (jarg < 1) then
  write(*,'("Usage: xpdb2hb pdbfile > hblist ")')
  stop 'pdb2hb.f90 Sat Nov  8 16:01:44 EST 2008'
endif

call getarg(1,pdbfile)

!---- announce
write(*,'("!-----------------------------------------------------!")')
write(*,'("! pdb2hb.f90 : This program extracts Hbonding infor-  !")')
write(*,'("!            mation to stdout                         !")')
write(*,'("!            C.Bystroff 8-NOV-2008                    !")')
write(*,'("!-----------------------------------------------------!")')
write(*,'("! donor acceptor H-or-S ")')
write(*,'("! pdb file = ",a)') trim(pdbfile)

  !! read PDB file
  call READPDB(pdbfile,xyz,nres,sg,nsg)
  !! find backbone H-bonds
  call GETHBONDS(xyz,nres)
  if (nsg/=0) call GETSSBONDS(sg,nres)

if (associated(xyz)) deallocate(xyz)
if (associated(sg)) deallocate(sg)

CONTAINS
!-----------------------------------------------------------------------!
subroutine GETHBONDS(xyz,nres)
  integer,intent(in) :: nres
  real,dimension(:,:),pointer :: xyz
  integer :: ires,jres,ios
  real,dimension(3) :: vec,vvv,hvec
  real :: dd
  real,parameter :: HBCUT=3.5, HOCUT=2.5
  if (.not.associated(xyz)) stop 'Bug in GETHBONDS 5'
  !! loop over all nitrogens
  do ires=1,nres
    if (xyz(1,(ires-1)*4+1)>998.) cycle
    do jres=1,nres
      if (ires==jres) cycle
      if (xyz(1,(jres-1)*4+4)>998.) cycle  
      vec = xyz(1:3,(ires-1)*4+1) - xyz(1:3,(jres-1)*4+4)  !! O -> N
      dd = sqrt(sum(vec*vec))
      if (dd > HBCUT) cycle
      !! For close oxygens, calculate amide H position, roughly
      if (ires > 1) then
        vvv = xyz(1:3,(ires-1)*4+1) - xyz(1:3,(ires-2)*4+3) + &
              xyz(1:3,(ires-1)*4+1) - xyz(1:3,(ires-1)*4+2)
        vvv = vvv/sqrt(sum(vvv*vvv))  !! length 1 vector. N-H bond is length 1.
        hvec = xyz(1:3,(ires-1)*4+1) + vvv
        vvv = hvec - xyz(1:3,(jres-1)*4+4)   !! O -> H
        dd = sqrt(sum(vvv*vvv))
        if (dd > HOCUT) cycle
      endif
      write(*,'(2i7," H ")') ires, jres
    enddo
  enddo
end subroutine GETHBONDS
!-----------------------------------------------------------------------!
!-----------------------------------------------------------------------!
subroutine GETSSBONDS(sg,nres)
  integer,intent(in) :: nres
  real,dimension(:,:),pointer :: sg
  integer :: ires,jres,ios,nss
  real,dimension(3) :: vec
  real :: dd
  real,parameter :: SSCUT=2.5
  if (.not.associated(sg)) stop 'Bug in GETSSBONDS 5'
  !! loop over all residues
  nss = 0
  do ires=1,nres
    if (sg(1,ires)==0.000) cycle
    do jres=ires+1,nres
      if (sg(1,jres)==0.000) cycle
      vec = sg(1:3,ires) - sg(1:3,jres)
      dd = sqrt(sum(vec*vec))
      if (dd > SSCUT) cycle
      nss = nss + 1
      write(*,'(2i7," S ")') ires, jres
      exit
    enddo
  enddo
  deallocate(sg)
end subroutine GETSSBONDS

!-----------------------------------------------------------------------!
subroutine READPDB(pdbfile,xyz,nres,sg,nsg)
implicit none
character(len=*),intent(in) :: pdbfile
real,dimension(:,:),pointer :: xyz
real,dimension(:,:),pointer :: sg
integer,intent(out) :: nres, nsg
integer :: jres,ires,lres
real :: b,p,chi1,chi2,x
integer :: I,J,K,L,i1,i2,ist,startres,ios
real,dimension(3) :: coords
character(len=80) ::  card,filename
character(len=4) ::  atomname
character(len=4) ::  acid
character(len=6) :: last
character(len=1) :: ocode
character(len=80),parameter :: pdbinfmt="(BZ,6x,5x,1x,a4,1x,a4,1x,i4,4x,3f8.3,2f6.2)"
!** initialization
last = '     '
jres = 0
nsg = 0
open(1,file=pdbfile,status='old',form='formatted',action='read',iostat=ios)
if (ios/=0) then
  write(0,*) 'Error opening PDB file:',trim(pdbfile)
  stop 'Error opening PDB file'
endif
ocode = ' '
jres = 0
do
  read(1,'(a80)',iostat=ios) card
  if (ios/=0) exit
  if (jres >  0.and.card(1:5) == 'ENDMD') exit
  if (jres >  0.and.card(1:5) == 'TER  ') exit
  if (card(1:5) /= 'ATOM ') cycle
  !! assumed clean PDB file ----
  ! if (pchain/='?'.and.card(22:22)/=pchain ) cycle
  ! if (pchain=='?') pchain = card(22:22)   !! if chain is unknown, use frst chain
  ! if (card(27:27)/=ocode ) cycle
  !! read(card,pdbinfmt,iostat=ios) atomname,acid,ires,coords,b,p
  !! if (ios/=0) cycle
  if (card(22:27) /= last) then  ! new residue starting
    jres = jres + 1
    last = card(22:27)
  endif
enddo
nres = jres
write(*,'("!",i9," residues in PDB file.")') nres
rewind(1)
if (associated(xyz)) deallocate(xyz)
allocate(xyz(3,nres*4),stat=ios); if (ios/=0) stop 'Failed to allocate xyz'
if (associated(sg)) deallocate(sg)
allocate(sg(3,nres),stat=ios); if (ios/=0) stop 'Failed to allocate sg'
sg = 0.000
last = '     '
jres = 0
ios = 0
xyz = 999.
do 
  read(1,'(a80)',iostat=ios) card
  if (ios/=0) exit
  if (jres >  0.and.card(1:5) == 'ENDMD') exit
  if (jres >  0.and.card(1:5) == 'TER  ') exit
  if (card(1:5) /= 'ATOM ') cycle
  ! if (card(22:22)/=pchain ) cycle
  ! if (card(27:27)/=ocode ) cycle
  read(card,pdbinfmt,iostat=ios) atomname,acid,ires,coords,b,p
  if (ios/=0) cycle
  if (card(22:27) /= last) then  ! new residue starting
    jres = jres + 1
    if (jres > nres) exit
    last = card(22:27)
    ! pdbreslist(jres) = ires
  endif
  select case (atomname)
    case (' N  ')
      xyz(1:3,(jres-1)*4+1) = coords
    case (' CA ')
      xyz(1:3,(jres-1)*4+2) = coords
    case (' C  ')
      xyz(1:3,(jres-1)*4+3) = coords
    case (' O  ')
      xyz(1:3,(jres-1)*4+4) = coords
    case (' SG ')
      nsg = nsg + 1
      sg(1:3,jres) = coords
    case default  ! ignore
  !! diagnostic
  !!    write(*,'("Nonbackbone atom: ",i4,">>>",a4,"<<<")') ires,atomname
  end select 
enddo
close(1)
end subroutine READPDB
!-----------------------------------------------------------------------!
!-----------------------------------------------------------------------!
!-----------------------------------------------------------------------!
!-----------------------------------------------------------------------!
end program pdb2hb
