program contactmask
!! Formerly unfold_energy
!! CB  Tue Apr 15 19:48:03 EDT 2008
use masker
implicit none
!!
! This program reads a PDB file and calculates the energy
! between each pair of residues in the sequence.
! The energy function is a linear combination of an Hbond
! term (Hbond) and a solvation term (SAS) .
!! 
character(len=200) :: xyzfile, aline,outfile
real,dimension(:,:),allocatable :: xyz,xyznew
integer,dimension(:),allocatable :: saveatype
integer :: i,j,k,nseed,jj,n,nn,isize,jsize,ios=0
real :: tt, sig=0.3,x,y,z,nrg,scl,xmin,xmax,ymin,ymax,zmin,zmax
real :: pairwise,gsolv,sasa
real,dimension(:),allocatable :: residuesasa,buried,residueGsolv
real,dimension(3) :: vec
integer :: iargc, jarg, time, ounit=22, nhet, nat, nres, ires, jres
logical :: drawingatall = .false.
character(len=11) :: lastresstring
character :: altloc
integer,dimension(:),allocatable :: nposit
! integer, dimension(:), pointer::atype  ! public
!! set default values and constants --------------------------------------------
xmin = 999.; ymin = 999.; zmin = 999.
xmax = -999.; ymax = -999.; zmax = -999.
!! rendering = .false.
!! smoothing = .false.
!! saying = .false.
lastresstring="           "
!! Read command line ----------------------------------
nseed = time()
jarg = iargc()
!! lbox = 0.00  !! masker global
if (jarg < 1) then
  write(*,'("Usage: xcontactmask PDBfile [output proberadius]")')
  stop 'contactmask.f90 v.  Sat Jan 21 10:19:42 EST 2006'
endif
call getarg(1,xyzfile)
if (jarg >=2 ) then
  call getarg(2,outfile)
  ounit = 22
  open(ounit,file=outfile,status='replace',form='formatted',iostat=ios)
  if (ios/=0) stop 'contactmask:: error attempting to open file. wrong permissions?'
else
  ounit = 6
endif
if (jarg >=3 ) then
  call getarg(3,aline)
  read(aline,*,iostat=ios) rw
  if (ios/=0) stop 'Bad value for rw'
  write(*,'("Probe radius: ",f8.2)') rw
endif
!! Initialize masks
call masker_plot(drawingatall)
call masker_initmasks()
! call masker_getatypeptr(atype)
!!-------- read atom coordinates from file, just count them
open(12,file=xyzfile,status='old',form='formatted')
i = 0
j = 0
nres = 0
do
  read(12,'(a)',iostat=ios) aline
  if (ios/=0) exit
  if (aline(1:5)/='ATOM '.and.aline(1:6)/='HETATM') cycle
  if (aline(17:17)/=" ") then
    write(*,'("WARNING: alt loc indicator detected!!")')
    if (altloc==" ") altloc = aline(17:17)
    if (aline(17:17)==altloc) then
      aline(17:17)=" "
    endif
  else
    altloc = " "
  endif
  if (aline(17:17)/=" ") cycle
  if (aline(1:5)=='ATOM ') i = i + 1
  if (aline(1:6)=='HETATM') j = j + 1
  if (lastresstring/=aline(17:27)) then
    nres = nres + 1
    lastresstring = aline(17:27)
    ! write(*,'("Residue ",i6," ",a6," ",a11)') nres,aline(1:6),lastresstring
  endif
enddo
nat = i
nhet = j
!! diagnostic
write(*,*) 'Number of atoms:', nat
write(*,*) 'Number of hetatoms:', nhet
write(*,*) 'Number of residues:', nres
nat = nat + nhet
!!-------- allocate arrays
allocate(xyz(3,nat),stat=ios)
if (ios/=0) stop 'Error allocating xyz'
allocate(xyznew(3,nat),stat=ios)
if (ios/=0) stop 'Error allocating xyznew'
allocate(saveatype(nat),stat=ios)
if (ios/=0) stop 'Error allocating saveatype'
allocate(nposit(nres+1),stat=ios)
if (ios/=0) stop 'Error allocating nposit'
allocate(residuesasa(nres),residueGsolv(nres),stat=ios)
if (ios/=0) stop 'Error allocating residuesasa'
allocate(buried(nat),stat=ios)
if (ios/=0) stop 'Error allocating buried'
!!--------  allocate atype() and other arrays within MASKER
call masker_allocatoms(nat)
write(*,*) 'Total atoms allocated:', nat
!!-------- rewind, read atom coordinates from file
rewind(12)
i = 0
ires = 0
vec = 0.
lastresstring = "           "
altloc = " "
do
  read(12,'(a)',iostat=ios) aline
  if (ios/=0) exit
  if ((aline(1:5)=='ATOM ').or.(aline(1:6)=='HETATM')) then
    if (aline(17:17)/=" ") then
      if (altloc==" ") altloc = aline(17:17)
      if (aline(17:17)==altloc) then
        aline(17:17)=" "
      endif
    else
      altloc = " "
    endif
    if (aline(17:17)/=" ") cycle
    i = i + 1
    read(aline(31:54),'(3f8.3)',iostat=ios) xyz(1:3,i)
    if (ios/=0) stop 'Format error xyzfile'
    if (lastresstring/=aline(17:27)) then
      ires = ires + 1
      nposit(ires) = i
      lastresstring = aline(17:27)
    endif
    jj = 0
    do j=1,nattype
      if (aline(14:17)==atomlib(j)%name(1:4)) then  !! allocated in MASKER::initmasks
        jj = j
        exit
      endif
    enddo
    if (jj==0) then
      write(*,'("Unknown atom type: ",a4,"  this atom will be ignored!")') aline(14:17)
      saveatype(i) = -1   !! dont use this atom
    else
      saveatype(i) = jj
    endif
    !! diagnostic
    !! write(*,*) 'Atom ',i,jj,xyz(1:3,i)
    vec = vec + xyz(1:3,i)
  endif
enddo
nposit(nres+1) = i+1
if (any(nposit(1:nres+1)==0)) stop 'BUG BUG BUG: nposit == 0'
!!-------- mov atoms to the center of mass
vec = -vec/real(nat)
do i=1,nat
  xyz(1:3,i) = xyz(1:3,i) + vec
enddo
!!-------- get SASA for individual residues
do ires=1,nres
  isize = nposit(ires+1) - nposit(ires)
  xyznew(1:3,1:isize) = xyz(1:3,nposit(ires):nposit(ires+1)-1)
  atype(1:isize) = saveatype(nposit(ires):nposit(ires+1)-1)
  !write(0,'("Calculating ",i9,": size=",i5," ",$)') ires,isize
  !write(0,*) xyznew(1:3,1:isize)
  !write(0,*) atype(1:isize) 
  call masker_getms(xyznew,isize,x,buried=buried,SAS=.true.,ses=sasa)
  !! ignore x, uses global varable sasa
  residuesasa(ires) = sasa
  residueGsolv(ires) = x
  !write(*,'(f9.3)') sasa
enddo
!!-------- get SASA for residue pairs and subtract from the sum of individual residue SASA.
do ires=1,nres-1
  isize = nposit(ires+1) - nposit(ires)
  xyznew(1:3,1:isize) = xyz(1:3,nposit(ires):nposit(ires+1)-1)
  atype(1:isize) = saveatype(nposit(ires):nposit(ires+1)-1)
  do jres=ires+1,nres
    jsize = nposit(jres+1) - nposit(jres)
    xyznew(1:3,isize+1:isize+jsize) = xyz(1:3,nposit(jres):nposit(jres)+jsize-1)
    atype(isize+1:isize+jsize) = saveatype(nposit(jres):nposit(jres)+jsize-1)
    jsize = jsize + isize
    call masker_getms(xyznew,jsize,x,buried=buried,SAS=.true.,ses=sasa)
    pairwise = residuesasa(ires) + residuesasa(jres) - sasa
    !! This is the energy of unfolding, not folding.
    gsolv = residueGsolv(ires) + residueGsolv(jres) - x
    if (abs(pairwise) > 0.01) then
      write(ounit,'(2i6,2f9.3)') ires,jres,pairwise,gsolv
    endif
  enddo
enddo
deallocate(xyz,stat=ios)
deallocate(xyznew,stat=ios)
deallocate(saveatype,stat=ios)
deallocate(nposit,stat=ios)
deallocate(residuesasa,stat=ios)
deallocate(buried,stat=ios)
call masker_deallo()
if (ounit/=6) close(ounit)
stop 'contactmask.f90  v.  Sat Jan 21 13:25:39 EST 2006'
! CONTAINS
!----------------------------------------------------!
end program contactmask
